@@ -19,7 +19,7 @@
 #include <objbase.h>
 #include <initguid.h>
 
-#include <QtGlobal>
+#include <KLocalizedString>
 
 #include "class_factory.h"
 
@@ -153,7 +153,7 @@ DllUnregisterServer() {
 
 SERVER* SERVER::instance()
 {
-   if(_instance == 0)
+   if(_instance == nullptr)
    {
       _instance = new SERVER();
       _instance->initLogging();
