@@ -5,7 +5,7 @@
  * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
  * SPDX-License-Identifier: GPL-2.0-or-later
 */
-//clazy:excludeall=connect-non-signal,incorrect-emit
+
 
 #include "MergeFileInfos.h"
 
@@ -18,8 +18,6 @@
 
 #include <KLocalizedString>
 
-QSharedPointer<DirectoryInfo> MergeFileInfos::m_dirInfo;
-
 MergeFileInfos::MergeFileInfos()
 {
     m_bEqualAB = false;
@@ -37,7 +35,6 @@ MergeFileInfos::MergeFileInfos()
     m_pFileInfoA = nullptr;
     m_pFileInfoB = nullptr;
     m_pFileInfoC = nullptr;
-    m_dirInfo.clear();
 }
 
 MergeFileInfos::~MergeFileInfos()
@@ -155,7 +152,7 @@ QString MergeFileInfos::fullNameA() const
     if(existsInA())
         return getFileInfoA()->absoluteFilePath();
 
-    return m_dirInfo->dirA().absoluteFilePath() + '/' + subPath();
+    return gDirInfo->dirA().absoluteFilePath() + '/' + subPath();
 }
 
 QString MergeFileInfos::fullNameB() const
@@ -163,7 +160,7 @@ QString MergeFileInfos::fullNameB() const
     if(existsInB())
         return getFileInfoB()->absoluteFilePath();
 
-    return m_dirInfo->dirB().absoluteFilePath() + '/' + subPath();
+    return gDirInfo->dirB().absoluteFilePath() + '/' + subPath();
 }
 
 QString MergeFileInfos::fullNameC() const
@@ -171,7 +168,7 @@ QString MergeFileInfos::fullNameC() const
     if(existsInC())
         return getFileInfoC()->absoluteFilePath();
 
-    return m_dirInfo->dirC().absoluteFilePath() + '/' + subPath();
+    return gDirInfo->dirC().absoluteFilePath() + '/' + subPath();
 }
 
 void MergeFileInfos::sort(Qt::SortOrder order)
@@ -184,12 +181,12 @@ void MergeFileInfos::sort(Qt::SortOrder order)
 
 QString MergeFileInfos::fullNameDest() const
 {
-    if(m_dirInfo->destDir().prettyAbsPath() == m_dirInfo->dirC().prettyAbsPath())
+    if(gDirInfo->destDir().prettyAbsPath() == gDirInfo->dirC().prettyAbsPath())
         return fullNameC();
-    else if(m_dirInfo->destDir().prettyAbsPath() == m_dirInfo->dirB().prettyAbsPath())
+    else if(gDirInfo->destDir().prettyAbsPath() == gDirInfo->dirB().prettyAbsPath())
         return fullNameB();
     else
-        return m_dirInfo->destDir().absoluteFilePath() + '/' + subPath();
+        return gDirInfo->destDir().absoluteFilePath() + '/' + subPath();
 }
 
 bool MergeFileInfos::compareFilesAndCalcAges(QStringList& errors, QSharedPointer<Options> const &pOptions, DirectoryMergeWindow* pDMW)
