@@ -1,22 +1,10 @@
-/**
- * Copyright (C) 2002-2009  Joachim Eibl, joachim.eibl at gmx.de
- * Copyright (C) 2018 Michael Reeves <reeves.87@gmail.com>
- *
- * This file is part of KDiff3.
- *
- * KDiff3 is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
- *
- * KDiff3 is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with KDiff3.  If not, see <http://www.gnu.org/licenses/>.
- */
+/*
+ * KDiff3 - Text Diff And Merge Tool
+ * 
+ * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
+ * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
+ * SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #ifndef OPTIONITEMS_H
 #define OPTIONITEMS_H
@@ -40,12 +28,13 @@ class OptionItemBase
         m_saveName = saveName;
         m_bPreserved = false;
     }
-    virtual ~OptionItemBase(){};
+    virtual ~OptionItemBase() = default;
     virtual void setToDefault() = 0;
     virtual void setToCurrent() = 0;
 
     virtual void apply() = 0;
-    virtual void write(ValueMap*) = 0;
+    
+    virtual void write(ValueMap*) const = 0;
     virtual void read(ValueMap*) = 0;
     void doPreserve()
     {
@@ -62,8 +51,7 @@ class OptionItemBase
             unpreserve();
         }
     }
-    QString getSaveName() { return m_saveName; }
-
+    QString getSaveName() const { return m_saveName; }
   protected:
     virtual void preserve() = 0;
     virtual void unpreserve() = 0;
@@ -109,7 +97,7 @@ class Option : public OptionItemBase
     void apply() override {};
     virtual void apply(const T& inValue) { *m_pVar = inValue; }
 
-    void write(ValueMap* config) override { config->writeEntry(m_saveName, *m_pVar); }
+    void write(ValueMap* config) const override { config->writeEntry(m_saveName, *m_pVar); }
     void read(ValueMap* config) override { *m_pVar = config->readEntry(m_saveName, m_defaultVal); }
 
   protected:
