@@ -335,7 +335,8 @@ QTextCodec* SourceData::detectEncoding(const QString& fileName, QTextCodec* pFal
     QFile f(fileName);
     if(f.open(QIODevice::ReadOnly))
     {
-        char buf[200];
+        char buf[400];
+
         qint64 size = f.read(buf, sizeof(buf));
         qint64 skipBytes = 0;
         QTextCodec* pCodec = detectEncoding(buf, size, skipBytes);
@@ -583,7 +584,7 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
         return false;
 
     QString line;
-    QChar curChar;
+    QChar curChar, prevChar = 0;
     LineCount lineCount = 0;
     qint64 lastOffset = 0;
     qint64 skipBytes = 0;
@@ -609,6 +610,7 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
     m_unicodeBuf->clear();
     Q_ASSERT(m_unicodeBuf->length() == 0);
 
+    bool skipNextRead = false;
     while(!ts.atEnd())
     {
         line.clear();
@@ -618,7 +620,12 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
             return false;
         }
 
-        curChar = ts.read(1).unicode()[0];
+        if(!skipNextRead){
+            prevChar = curChar;
+            curChar = ts.read(1).unicode()[0];
+        }
+        else
+            skipNextRead = false;
 
         quint32 firstNonwhite = 0;
         bool    foundNonWhite = false;
@@ -645,6 +652,7 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
             if(ts.atEnd())
                 break;
 
+            prevChar = curChar;
             curChar = ts.read(1).unicode()[0];
         }
 
@@ -657,20 +665,16 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
             case '\r':
                 if(lastOffset < mDataSize)
                 {
-                    //workaround for lack of peak API in QTextStream.
-                    qint64 j;
-                    for(j = 0; j < 4 && lastOffset + j < mDataSize; ++j)
-                    {
-                        if(m_pBuf[ts.pos() + j] != '\0')
-                            break;
-                    }
+                    prevChar = curChar;
+                    curChar = ts.read(1).unicode()[0];
 
-                    if(m_pBuf[ts.pos() + j] == '\n')
+                    if(curChar == '\n')
                     {
-                        curChar = ts.read(1).unicode()[0];
                         vOrigDataLineEndStyle.push_back(eLineEndStyleDos);
                         break;
                     }
+                    //work around for lack of seek API in QTextStream
+                    skipNextRead = true;
                 }
 
                 //old mac style ending.
@@ -834,5 +838,18 @@ QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, qint64& ski
                 break;
         }
     }
+    //Attempt to detect non-bom UTF8. This is a very common encoding.
+    return dectectUTF8(s);
+}
+
+QTextCodec* SourceData::dectectUTF8(const QByteArray& data)
+{
+    QTextCodec* utf8 = QTextCodec::codecForName("UTF-8");
+
+    QTextCodec::ConverterState state;
+    const QString validText = utf8->toUnicode(data.constData(), data.size(), &state);
+    if(state.invalidChars == 0)
+        return utf8;
+
     return nullptr;
 }
