@@ -58,10 +58,11 @@ bool LineData::equal(const LineData& l1, const LineData& l2)
     if(g_bIgnoreWhiteSpace)
     {
         // Ignore white space diff
-        QString::const_iterator p1 = l1.getLine().begin();
-        QString::const_iterator p1End = l1.getLine().end();
-        QString::const_iterator p2 = l2.getLine().begin();
-        QString::const_iterator p2End = l2.getLine().end();
+        const QString line1 = l1.getLine(), line2 = l2.getLine();
+        QString::const_iterator p1 = line1.begin();
+        QString::const_iterator p1End = line1.end();
+        QString::const_iterator p2 = line2.begin();
+        QString::const_iterator p2End = line2.end();
 
         for(; p1 != p1End && p2 != p2End; p1++, p2++)
         {
@@ -76,7 +77,8 @@ bool LineData::equal(const LineData& l1, const LineData& l2)
     }
     else
     {
-        return (l1.size() == l2.size() && QString::compare(l1.getLine(), l2.getLine()) == 0);
+        const QString line1 = l1.getLine(), line2 = l2.getLine();
+        return (l1.size() == l2.size() && QString::compare(line1, line2) == 0);
     }
 }
 
@@ -1205,6 +1207,9 @@ void calcDiff(const QString& line1, const QString& line2, DiffList& diffList, in
 {
     diffList.clear();
 
+    const QChar* p1end = line1.constData() + line1.size();
+    const QChar* p2end = line2.constData() + line2.size();
+
     QString::const_iterator p1=line1.begin(), p2=line2.begin();
 
     /*
@@ -1230,19 +1235,19 @@ void calcDiff(const QString& line1, const QString& line2, DiffList& diffList, in
 
         for(i1 = 0;; ++i1)
         {
-            if(p1[i1] == *line1.end() || (bBestValid && i1 >= bestI1 + bestI2))
+            if(&p1[i1] == p1end || (bBestValid && i1 >= bestI1 + bestI2))
             {
                 break;
             }
             for(i2 = 0; i2 < maxSearchRange; ++i2)
             {
-                if(p2[i2] == *line2.end() || (bBestValid && i1 + i2 >= bestI1 + bestI2))
+                if(&p2[i2] == p2end || (bBestValid && i1 + i2 >= bestI1 + bestI2))
                 {
                     break;
                 }
                 else if(p2[i2] == p1[i1] &&
-                        (match == 1 || abs(i1 - i2) < 3 || (p2[i2 + 1] == *line2.end() && p1[i1 + 1] == *line1.end()) ||
-                         (p2[i2 + 1] != *line2.end() && p1[i1 + 1] != *line1.end() && p2[i2 + 1] == p1[i1 + 1])))
+                        (match == 1 || abs(i1 - i2) < 3 || (&p2[i2 + 1] == p2end && &p1[i1 + 1] == p1end) ||
+                         (&p2[i2 + 1] != p2end && &p1[i1 + 1] != p1end && p2[i2 + 1] == p1[i1 + 1])))
                 {
                     if(i1 + i2 < bestI1 + bestI2 || !bBestValid)
                     {
