@@ -1,31 +1,20 @@
-/**
- * Copyright (c) 2008 by Valentin Rusu kde at rusu.info
- * Copyright (C) 2018 Michael Reeves <reeves.87@gmail.com>
+/*
+ * KDiff3 - Text Diff And Merge Tool
  *
- * This file is part of KDiff3.
- *
- * KDiff3 is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
- *
- * KDiff3 is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with KDiff3.  If not, see <http://www.gnu.org/licenses/>.
- */
+ * SPDX-FileCopyrightText: 2008 Valentin Rusu kde at rusu.info
+ * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
+ * SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #ifndef GUIUTILS_H
 #define GUIUTILS_H
 
+#include "defmac.h"
 #include <QObject>
 #include <kactioncollection.h>
 
 namespace GuiUtils {
-   //Use std::enable_if since complires don't disabiguate overloads based on return type alone
+   //Use std::enable_if since compilers don't disabiguate overloads based on return type alone
    template <class T, class Receiver, class Func>
       inline typename std::enable_if<std::is_same<T, QAction>::value, QAction>::type* createAction(
        const QString& text,
@@ -39,7 +28,7 @@ namespace GuiUtils {
 
        theAction = ac->addAction(actionName);
        theAction->setText(text);
-       QObject::connect(theAction, &QAction::triggered, receiver, slot);
+       chk_connect(theAction, &QAction::triggered, receiver, slot);
        return theAction;
    }
 
@@ -54,7 +43,7 @@ namespace GuiUtils {
       KToggleAction* theAction = new KToggleAction(ac);
       ac->addAction( actionName, theAction );
       theAction->setText( text );
-      QObject::connect( theAction, &KToggleAction::triggered, receiver, slot );
+      chk_connect( theAction, &KToggleAction::triggered, receiver, slot );
       return theAction;
    }
 
