@@ -1,20 +1,18 @@
-/***************************************************************************
- *   Copyright (C) 2003-2007 by Joachim Eibl                               *
- *   joachim.eibl at gmx.de                                                *
- *   Copyright (C) 2019 Michael Reeves <reeves.87@gmail.com>               *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- ***************************************************************************/
+/*
+ * KDiff3 - Text Diff And Merge Tool
+ * 
+ * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
+ * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
+ * SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 
 #ifndef PROGRESS_H
 #define PROGRESS_H
 
 #include <QDialog>
-#include <QTime>
+#include <QPointer>
+#include <QElapsedTimer>
 #include <QList>
 
 class KJob;
@@ -60,8 +58,8 @@ public:
    void hideStatusBarWidget();
    void delayedHideStatusBarWidget();
 
-   void timerEvent(QTimerEvent*) override;
-public slots:
+   void timerEvent(QTimerEvent* event) override;
+public Q_SLOTS:
    void recalc(bool bUpdate);
 private:
 
@@ -84,7 +82,7 @@ private:
    int m_progressDelayTimer;
    int m_delayedHideTimer;
    int m_delayedHideStatusBarWidgetTimer;
-   QList<QEventLoop*> m_eventLoopStack;
+   QPointer<QEventLoop> m_eventLoop;
 
    QProgressBar* m_pProgressBar;
    QProgressBar* m_pSubProgressBar;
@@ -92,11 +90,11 @@ private:
    QLabel* m_pSubInformation;
    QLabel* m_pSlowJobInfo;
    QPushButton* m_pAbortButton;
-   QTime m_t1;
-   QTime m_t2;
+   QElapsedTimer m_t1;
+   QElapsedTimer m_t2;
    bool m_bWasCancelled;
    e_CancelReason m_eCancelReason;
-   KJob* m_pJob;
+   KJob* m_pJob = nullptr;
    QString m_currentJobInfo;  // Needed if the job doesn't stop after a reasonable time.
    bool m_bStayHidden;
    QThread* m_pGuiThread;
@@ -106,7 +104,7 @@ private:
    QPushButton* m_pStatusAbortButton;
 protected:
    void reject() override;
-private slots:
+private Q_SLOTS:
    void delayedHide();
    void slotAbort();
 };
